@@ -29,6 +29,7 @@
 #include <math.h>
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <glib-object.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <tumbler/tumbler.h>
@@ -40,6 +41,7 @@
 
 
 
+#define GUESS_BUFFER_SIZE           512
 #define BORING_IMAGE_VARIANCE       256.0    /* tweak this if necessary */
 #define TUMBLER_GST_PLAY_FLAG_VIDEO (1 << 0) /* from GstPlayFlags */
 #define TUMBLER_GST_PLAY_FLAG_AUDIO (1 << 1) /* from GstPlayFlags */
@@ -565,19 +567,63 @@ gst_thumbnailer_create (TumblerAbstractThumbnailer *thumbnailer,
   gint                    width, height;
   TumblerThumbnailFlavor *flavor;
   GdkPixbuf              *scaled;
+  const gchar            *uri;
+  GFile                  *file;
+  GFileInputStream       *stream;
+  guchar                  buffer[GUESS_BUFFER_SIZE];
+  gssize                  size;
+  gchar                  *guessed_type, *claimed_type;
+  gboolean                uncertain, equals;
 
   /* check for early cancellation */
   if (g_cancellable_is_cancelled (cancellable))
     return;
 
   /* Check if is a sparse video file */
+  uri = tumbler_file_info_get_uri (info);
   if (tumbler_util_guess_is_sparse (info))
   {
-    g_debug ("Video file '%s' is probably sparse, skipping\n",
-             tumbler_file_info_get_uri (info));
+    g_debug ("Video file '%s' is probably sparse, skipping\n", uri);
     return;
   }
 
+  /* check if real mime type matches advertised mime type to avoid server-side request
+   * forgery: see https://gitlab.xfce.org/xfce/tumbler/-/issues/65 */
+  file = g_file_new_for_uri (uri);
+  stream = g_file_read (file, cancellable, &error);
+  g_object_unref (file);
+  if (stream == NULL)
+    {
+      g_signal_emit_by_name (thumbnailer, "error", uri, error->code, error->message);
+      g_error_free (error);
+
+      return;
+    }
+
+  size = g_input_stream_read (G_INPUT_STREAM (stream), buffer, GUESS_BUFFER_SIZE,
+                              cancellable, &error);
+  g_object_unref (stream);
+  if (size == -1)
+    {
+      g_signal_emit_by_name (thumbnailer, "error", uri, error->code, error->message);
+      g_error_free (error);
+
+      return;
+    }
+
+  guessed_type = g_content_type_guess (NULL, buffer, size, &uncertain);
+  claimed_type = g_content_type_from_mime_type (tumbler_file_info_get_mime_type (info));
+  equals = guessed_type != NULL && claimed_type != NULL
+           && g_content_type_equals (guessed_type, claimed_type);
+  g_free (guessed_type);
+  g_free (claimed_type);
+  if (uncertain || ! equals)
+    {
+      g_signal_emit_by_name (thumbnailer, "error", uri, TUMBLER_ERROR_INVALID_FORMAT,
+                             _("Thumbnail could not be inferred from file contents"));
+      return;
+    }
+
   /* get size of dest thumb */
   thumbnail = tumbler_file_info_get_thumbnail (info);
   flavor = tumbler_thumbnail_get_flavor (thumbnail);
